import { wait } from './cypress-helper';

//TODO Naming der Methoden geradeziehen

export function containClass(element: any, cssClass: string): void {
  element.should('have.class', cssClass);
}

export function notContainClass(element: any, cssClass: string): void {
  element.should('not.have.class', cssClass);
}

export function exist(element: any): void {
  element.should('exist');
}

export function notExist(element: any): void {
  element.should('not.exist');
}

export function haveText(element: any, text: string): void {
  element
    .invoke('text')
    .then((elementText) => elementText.trim())
    .should('equal', text);
}

export function haveValue(element: any, value: string): void {
  element.should('have.value', value);
}

export function haveFocus(element: any): void {
  element.should('have.focus');
}

export function mouseEnter(element: any): void {
  element.trigger('mouseenter');
}

export function mouseOver(element: any): void {
  element.trigger('mouseover');
}

export function contains(element: any, containing: string): void {
  element.should('exist').contains(containing);
}

export function notContains(element: any, containing: string): void {
  element.contains(containing).should('not.exist');
}

export function haveLength(element: any, length: number): void {
  element.should('have.length', length);
}

export function beChecked(element: any): void {
  element.should('be.checked');
}

export function notBeChecked(element: any): void {
  element.should('not.be.checked');
}

//TODO: "first()" rausnehmen -> im html eine entprechende data-test-id ansprechen?! | trennen in "get" und "verify"
export function shouldFirstContains(element: any, containing: string) {
  element.first().should('exist').contains(containing);
}

export function shouldHaveAttributeBeGreaterThan(
  element: any,
  attributeName: string,
  value: number,
) {
  element.first().should('exist').invoke(attributeName).should('be.gt', value);
}

export function shouldHaveAttributeBeLowerThan(element: any, attributeName: string, value: number) {
  element.first().should('exist').invoke(attributeName).should('be.gt', value);
}
//

export function shouldHaveAttribute(element: any, name: string, value: string) {
  element.should('have.attr', name, value);
}

export function visible(element: any) {
  element.should('be.visible');
}

export function notBeVisible(element: any) {
  element.should('not.be.visible');
}

export function enter(element: any): void {
  element.clear().type(CypressKeyboardActions.ENTER);
}

export function enterWith(
  element: Cypress.Chainable<JQuery<HTMLElement>>,
  value: string,
  delayBeforeEnter: number = 200,
): void {
  element.clear().type(value);
  wait(delayBeforeEnter);
  element.type(CypressKeyboardActions.ENTER);
}

export function typeText(element: Cypress.Chainable<JQuery<HTMLElement>>, value: string): void {
  element.type(value);
}

export function backspaceOn(element: any): void {
  element.type(CypressKeyboardActions.BACKSPACE);
}

enum CypressKeyboardActions {
  ENTER = '{enter}',
  BACKSPACE = '{backspace}',
}
