/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { VorgangForwardingE2EComponent } from '../../../components/vorgang/vorgang-forward.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { exist, haveText, notExist } from '../../../support/cypress.util';
import { FORWARDING_INVALID_EMAIL } from '../../../support/data.util';
import { loginAsEmil } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

describe.skip('{TODO: need to fix with a mock} Vorgang forwarding fail', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const forwardingContainer: VorgangForwardingE2EComponent = vorgangPage.getForwardingContainer();

  const vorgang: VorgangE2E = createVorgang();

  before(() => {
    initVorgang(vorgang);

    loginAsEmil();

    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('after click on vorgang', () => {
    it('should show vorgang header', () => {
      vorgangList.getListItem(vorgang.name).getRoot().click();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should show forwarding', () => {
      exist(forwardingContainer.getRoot());
    });
  });

  describe('fill forward formular', () => {
    it('should show spinner with text on valid formular', () => {
      const forwardForumlar = forwardingContainer.getFormular();
      forwardForumlar.getZustaendigeStelle().clear().type(FORWARDING_INVALID_EMAIL);
      forwardForumlar.getPasswort().clear().type('validespasswort');

      forwardForumlar.forward();

      exist(forwardingContainer.getSpinner());
    });
  });

  describe('fail forwarding', () => {
    it('should have status ' + vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG], () => {
      waitForSpinnerToDisappear();

      haveText(
        vorgangPage.getVorgangDetailHeader().getStatus(),
        vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG],
      );
    });

    it('should show forward fail text', () => {
      exist(forwardingContainer.getForwardingErrorMessage());
    });

    it('should NOT show "mark as success" button', () => {
      notExist(forwardingContainer.getMarkAsSuccessButton());
    });

    it('should show formular', () => {
      exist(forwardingContainer.getFormular().getRoot());
    });
  });
});
