import { ApiRootService } from '@alfa-client/api-root-shared';
import { Environment, ENVIRONMENT_CONFIG } from '@alfa-client/environment-shared';
import { NavigationSharedModule } from '@alfa-client/navigation-shared';
import { ResourceRepository, TechSharedModule } from '@alfa-client/tech-shared';
import { UiModule } from '@alfa-client/ui';
import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { ReactiveFormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import KcAdminClient from '@keycloak/keycloak-admin-client';
import { ButtonWithSpinnerComponent, TextareaEditorComponent } from '@ods/component';
import {
  ExclamationIconComponent,
  ListComponent,
  ListItemComponent,
  MailboxIconComponent,
  PersonIconComponent,
  TextInputComponent,
} from '@ods/system';
import { createSettingListResourceService, SettingListResourceService } from './admin-settings-resource.service';
import { SettingsService } from './admin-settings.service';
import { ConfigurationResourceService, createConfigurationResourceService } from './configuration/configuration-resource.service';
import { ConfigurationService } from './configuration/configuration.service';
import {
  createOrganisationsEinheitListResourceService,
  OrganisationsEinheitListResourceService,
} from './organisationseinheit/organisations-einheit-list-resource.service';
import {
  createOrganisationsEinheitResourceService,
  OrganisationsEinheitResourceService,
} from './organisationseinheit/organisations-einheit-resource.service';
import { OrganisationsEinheitContainerComponent } from './organisationseinheit/organisationseinheit-container/organisationseinheit-container.component';
import { OrganisationsEinheitListComponent } from './organisationseinheit/organisationseinheit-container/organisationseinheit-list/organisationseinheit-list.component';
import { OrganisationsEinheitFormContainerComponent } from './organisationseinheit/organisationseinheit-form-container/organisationseinheit-form-container.component';
import { OrganisationsEinheitFormComponent } from './organisationseinheit/organisationseinheit-form-container/organisationseinheit-form/organisationseinheit-form.component';
import { OrganisationsEinheitSignaturComponent } from './organisationseinheit/organisationseinheit-form-container/organisationseinheit-form/organisationseinheit-signatur/organisationseinheit-signatur.component';
import { PostfachContainerComponent } from './postfach/postfach-container/postfach-container.component';
import { PostfachFormComponent } from './postfach/postfach-container/postfach-form/postfach-form.component';
import { PostfachSignaturComponent } from './postfach/postfach-container/postfach-form/postfach-signatur/postfach-signatur.component';
import { createPostfachResourceService, PostfachResourceService } from './postfach/postfach-resource.service';
import { PostfachService } from './postfach/postfach.service';
import { MoreItemButtonComponent } from './shared/more-menu/more-item-button/more-item-button.component';
import { MoreMenuComponent } from './shared/more-menu/more-menu.component';
import { NavigationItemComponent } from './shared/navigation-item/navigation-item.component';
import { PrimaryButtonComponent } from './shared/primary-button/primary-button.component';
import { SecondaryButtonComponent } from './shared/secondary-button/secondary-button.component';
import { SpinnerComponent } from './shared/spinner/spinner.component';
import { TextFieldComponent } from './shared/text-field/text-field.component';
import { ToUserNamePipe } from './user/to-user-name.pipe';
import { UsersRolesComponent } from './users-roles/users-roles.component';

@NgModule({
  declarations: [
    PostfachContainerComponent,
    PostfachFormComponent,
    PostfachSignaturComponent,
    NavigationItemComponent,
    TextFieldComponent,
    OrganisationsEinheitContainerComponent,
    OrganisationsEinheitListComponent,
    OrganisationsEinheitFormContainerComponent,
    OrganisationsEinheitFormComponent,
    OrganisationsEinheitSignaturComponent,
    PrimaryButtonComponent,
    SecondaryButtonComponent,
    MoreMenuComponent,
    MoreItemButtonComponent,
    SpinnerComponent,
    UsersRolesComponent,
  ],
  imports: [
    CommonModule,
    TechSharedModule,
    RouterModule,
    ReactiveFormsModule,
    TextInputComponent,
    ButtonWithSpinnerComponent,
    TextareaEditorComponent,
    MailboxIconComponent,
    PersonIconComponent,
    ToUserNamePipe,
    ListComponent,
    ListItemComponent,
    ExclamationIconComponent,
    UiModule,
    NavigationSharedModule,
  ],
  exports: [
    PostfachContainerComponent,
    OrganisationsEinheitContainerComponent,
    OrganisationsEinheitFormContainerComponent,
    NavigationItemComponent,
    UsersRolesComponent,
  ],
  providers: [
    ConfigurationService,
    SettingsService,
    PostfachService,
    {
      provide: KcAdminClient,
      useFactory: (env: Environment) =>
        new KcAdminClient({
          baseUrl: env.authServer,
          realmName: env.realm,
        }),
      deps: [ENVIRONMENT_CONFIG],
    },
    {
      provide: PostfachResourceService,
      useFactory: createPostfachResourceService,
      deps: [ResourceRepository, SettingsService],
    },
    {
      provide: ConfigurationResourceService,
      useFactory: createConfigurationResourceService,
      deps: [ResourceRepository, ApiRootService],
    },
    {
      provide: SettingListResourceService,
      useFactory: createSettingListResourceService,
      deps: [ResourceRepository, ConfigurationService],
    },
    {
      provide: OrganisationsEinheitListResourceService,
      useFactory: createOrganisationsEinheitListResourceService,
      deps: [ResourceRepository, ApiRootService],
    },
    {
      provide: OrganisationsEinheitResourceService,
      useFactory: createOrganisationsEinheitResourceService,
      deps: [ResourceRepository, OrganisationsEinheitListResourceService],
    },
  ],
})
export class AdminSettingsModule {}
