import { AdminOrganisationsEinheitListResource, OrganisationsEinheitContainerComponent } from '@admin-client/admin-settings';
import { StateResource, ToEmbeddedResourcesPipe, createStateResource } from '@alfa-client/tech-shared';
import { Mock, existsAsHtmlElement, getMockComponent, mock } from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { OrganisationsEinheitListResource } from '@alfa-client/zustaendige-stelle-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { createAdminOrganisationsEinheitListResource } from '../../../../test/organisations-einheit/organisations-einheit';
import { OrganisationsEinheitService } from '../organisationseinheit.service';
import { OrganisationsEinheitListComponent } from './organisationseinheit-list/organisationseinheit-list.component';

describe('OrganisationsEinheitContainerComponent', () => {
  let component: OrganisationsEinheitContainerComponent;
  let fixture: ComponentFixture<OrganisationsEinheitContainerComponent>;
  let organisationsEinheitService: Mock<OrganisationsEinheitService>;

  const organisationsEinheitListStateResource: StateResource<AdminOrganisationsEinheitListResource> = createStateResource(
    createAdminOrganisationsEinheitListResource(),
  );

  const listSelector: string = getDataTestIdOf('organisations-einheit-list');

  beforeEach(async () => {
    organisationsEinheitService = {
      ...mock(OrganisationsEinheitService),
      getList: jest.fn().mockReturnValue(of(organisationsEinheitListStateResource)),
    };

    await TestBed.configureTestingModule({
      declarations: [
        OrganisationsEinheitContainerComponent,
        MockComponent(OrganisationsEinheitListComponent),
        MockComponent(SpinnerComponent),
        ToEmbeddedResourcesPipe,
      ],
      imports: [ButtonWithSpinnerComponent],
      providers: [{ provide: OrganisationsEinheitService, useValue: organisationsEinheitService }],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationsEinheitContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('ngOnInit', () => {
      it('should call organisationsEinheitService getList', (done) => {
        component.ngOnInit();

        component.organisationsEinheitListStateResource$.subscribe(() => {
          expect(organisationsEinheitService.getList).toHaveBeenCalled();
          done();
        });
      });

      it('should set organisationsEinheitListStateResource$', (done) => {
        component.ngOnInit();

        component.organisationsEinheitListStateResource$.subscribe(
          (listStateResource: StateResource<OrganisationsEinheitListResource>) => {
            expect(listStateResource).toBe(organisationsEinheitListStateResource);
            done();
          },
        );
      });
    });
  });

  describe('template', () => {
    describe('organisationsEinheiten list', () => {
      it('should show list', () => {
        existsAsHtmlElement(fixture, listSelector);
      });

      it('should have organisationsEinheitResources', () => {
        const listComponent: OrganisationsEinheitListComponent = getMockComponent(fixture, OrganisationsEinheitListComponent);
        expect(listComponent.organisationsEinheitResources).toBeTruthy();
      });
    });
  });
});
