import { createEmptyStateResource, StateResource } from '@alfa-client/tech-shared';
import { SnackBarService } from '@alfa-client/ui';
import { Injectable } from '@angular/core';
import { Observable, startWith, tap } from 'rxjs';
import { SettingName } from '../admin-settings.model';
import { PostfachResourceService } from './postfach-resource.service';
import { Postfach, PostfachResource, PostfachSettingsItem } from './postfach.model';

@Injectable()
export class PostfachService {
  constructor(
    private postfachResourceService: PostfachResourceService,
    private snackbarService: SnackBarService,
  ) {}

  public get(): Observable<StateResource<PostfachResource>> {
    return this.postfachResourceService.get();
  }

  public save(postfach: Postfach): Observable<StateResource<PostfachResource>> {
    return this.postfachResourceService.save(this.buildPostfachSettingItem(postfach)).pipe(
      tap((stateResource: StateResource<PostfachResource>) =>
        this.showInfoAfterSave(stateResource),
      ),
      startWith(createEmptyStateResource<PostfachResource>(true)),
    );
  }

  private showInfoAfterSave(stateResource: StateResource<PostfachResource>) {
    if (!stateResource.loading) {
      this.snackbarService.showInfo('Die Signatur wurde erfolgreich gespeichert.');
    }
  }

  private buildPostfachSettingItem(postfach: Postfach): PostfachSettingsItem {
    return {
      name: SettingName.POSTFACH,
      settingBody: postfach,
    };
  }
}
