/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiDownloadToken } from '@alfa-client/api-root-shared';
import { BinaryFileLinkRel, BinaryFileResource } from '@alfa-client/binary-file-shared';
import { FileSizePipe, HasLinkPipe, createEmptyStateResource } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { IconButtonWithSpinnerComponent, SpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed, fakeAsync, tick } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MatTooltipModule } from '@angular/material/tooltip';
import { faker } from '@faker-js/faker';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent, MockModule } from 'ng-mocks';
import { BinaryFileComponent } from './binary-file.component';

describe('BinaryFileComponent', () => {
  let component: BinaryFileComponent;
  let fixture: ComponentFixture<BinaryFileComponent>;

  const file: BinaryFileResource = createBinaryFileResource([BinaryFileLinkRel.DOWNLOAD]);
  const downloadButton: string = getDataTestClassOf('download-file-button');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BinaryFileComponent,
        MatIcon,
        FileSizePipe,
        HasLinkPipe,
        MockModule(MatTooltipModule),
        MockComponent(SpinnerComponent),
        MockComponent(IconButtonWithSpinnerComponent),
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(BinaryFileComponent);
    component = fixture.componentInstance;
    component.file = file;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('Button', () => {
    it('should NOT disabled initial', () => {
      component.stateResource = createEmptyStateResource();
      fixture.detectChanges();

      const button = getElementFromFixture(fixture, downloadButton);

      expect(button).not.toBeDisabled();
    });

    it('should disabled if loading', () => {
      component.stateResource = createEmptyStateResource(true);
      fixture.detectChanges();

      const button = getElementFromFixture(fixture, downloadButton);

      expect(button).toBeDisabled();
    });
  });

  describe('click on download button', () => {
    it('should emit download if link exists', () => {
      jest.spyOn(component.startDownload, 'emit');
      component.file = createBinaryFileResource([BinaryFileLinkRel.DOWNLOAD]);

      component.downloadFile();

      expect(component.startDownload.emit).toHaveBeenCalledWith(component.file);
    });
  });

  describe('click and hold download button', () => {
    it('should call getNewDownloadToken', () => {
      jest.spyOn(component.getDownloadToken, 'emit');
      component.file = createBinaryFileResource();

      component.getNewDownloadToken();
      //TODO: Test mit onmousedown
      //const button: HTMLElement = getElementFromFixture(fixture, buttonSelector);
      //const mousedown = new MouseEvent('mousedown');
      //button.dispatchEvent(mousedown);

      expect(component.getDownloadToken.emit).toHaveBeenCalled();
    });
  });

  describe('click on delete button', () => {
    it('should emit delete', () => {
      jest.spyOn(component.startDelete, 'emit');
      component.file = createBinaryFileResource();

      component.deleteFile();

      expect(component.startDelete.emit).toHaveBeenCalledWith(component.file);
    });
  });

  describe('create download url', () => {
    it('should return a DownloadURL', () => {
      component.downloadToken = { token: faker.string.alpha({ length: 30 }) } as ApiDownloadToken;
      const pattern = `^[^:]+:[^:]+:http.+\\?token=${component.downloadToken.token}$`;
      const regex = new RegExp(pattern);

      const downloadUrl = component.createDownloadUrl();

      expect(downloadUrl).toMatch(regex);
    });
  });

  describe('handle spinner for drag end', () => {
    it('should set state resource loading', () => {
      component.handleSpinnerForDragEnd();

      expect(component.stateResource.loading).toBeTruthy();
    });

    it('should unset state resource loading', fakeAsync(() => {
      component.handleSpinnerForDragEnd();

      tick(3000);

      expect(component.stateResource.loading).toBeFalsy();
    }));
  });
});
