import { QueryList } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonToggleComponent } from '@ods/system';
import { ButtonToggleGroupComponent } from './button-toggle-group.component';

describe('ButtonToggleGroupComponent', () => {
  let component: ButtonToggleGroupComponent;
  let fixture: ComponentFixture<ButtonToggleGroupComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ButtonToggleGroupComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(ButtonToggleGroupComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('ngAfterContentInit', () => {
      beforeEach(() => {
        component.passPropsToChild = jest.fn();
      });

      it('should pass props to children', () => {
        const queryList: QueryList<ButtonToggleComponent> = new QueryList();
        queryList.reset([new ButtonToggleComponent(), new ButtonToggleComponent()]);
        component.buttonToggles = queryList;

        component.ngAfterContentInit();

        expect(component.passPropsToChild).toHaveBeenCalledTimes(2);
      });
    });

    describe('passPropsToChild', () => {
      const funcOne: () => number = jest.fn().mockReturnValue(1);
      const funcTwo: () => number = jest.fn().mockReturnValue(2);

      beforeEach(() => {
        component.setFormControlValue = jest.fn().mockReturnValue(funcOne);
        component.isValueSelected = jest.fn().mockReturnValue(funcTwo);
      });

      it('should call setFormControlValue', () => {
        const buttonToggle = new ButtonToggleComponent();
        buttonToggle.value = 'test';

        component.passPropsToChild(buttonToggle);

        expect(component.setFormControlValue).toHaveBeenCalledWith('test');
      });

      it('should set onClick for child', () => {
        const buttonToggle = new ButtonToggleComponent();

        component.passPropsToChild(buttonToggle);

        expect(buttonToggle.onClick).toBe(funcOne);
      });

      it('should call isValueSelected', () => {
        const buttonToggle = new ButtonToggleComponent();

        component.passPropsToChild(buttonToggle);

        expect(component.isValueSelected).toHaveBeenCalled();
      });

      it('should set isSelected for child', () => {
        const buttonToggle = new ButtonToggleComponent();

        component.passPropsToChild(buttonToggle);

        expect(buttonToggle.isSelected).toBe(funcTwo);
      });
    });

    describe('setFormControlValue', () => {
      beforeEach(() => {
        component.fieldControl.setValue = jest.fn();
        fixture.detectChanges();
      });

      it('should return function', () => {
        const result: Function = component.setFormControlValue('test');

        expect(typeof result).toBe('function');
      });

      it('should set value if values are not equal', () => {
        component.setFormControlValue('test')();

        expect(component.fieldControl.setValue).toHaveBeenCalledWith('test');
      });

      it('should not set value if values are equal', () => {
        component.setFormControlValue(null)();

        expect(component.fieldControl.setValue).not.toHaveBeenCalled();
      });
    });

    describe('isValueSelected', () => {
      it('should return function', () => {
        const result: Function = component.isValueSelected();

        expect(typeof result).toBe('function');
      });

      it('should return true', () => {
        const result: boolean = component.isValueSelected()(null);

        expect(result).toBe(true);
      });

      it('should return false', () => {
        const result: boolean = component.isValueSelected()('whatever');

        expect(result).toBe(false);
      });
    });
  });
});
