/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangResource, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createHistorieListResource } from 'libs/historie-shared/test/historie';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { HistorieListResource } from './historie.model';
import { HistorieService } from './historie.service';

describe('HistorieService', () => {
  let service: HistorieService;
  let resourceFactory = mock(ResourceFactory);
  let resourceWrapper = { get: jest.fn() };

  beforeEach(() => {
    service = new HistorieService(useFromMock(resourceFactory));

    resourceFactory.from.mockReturnValue(resourceWrapper);
  });

  describe('loadHistorieList', () => {
    const historieListResource: HistorieListResource = createHistorieListResource();
    const vorgang: VorgangResource = createVorgangResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: historieListResource }));
    });

    it('should call resourceFactory with given resource', () => {
      service.loadAll(vorgang);

      expect(resourceFactory.from).toHaveBeenCalledWith(vorgang);
    });

    it('should call resourceWrapper with link', () => {
      service.loadAll(vorgang);

      expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangWithEingangLinkRel.HISTORIE);
    });

    it('should return result', () => {
      const result = service.loadAll(vorgang);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(cold('a', { a: historieListResource }));
    });
  });
});
