import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { MockComponent } from 'ng-mocks';
import { ExpansionPanelWithUserComponent } from '../expansion-panel-with-user/expansion-panel-with-user.component';
import { HistorieItemAttachmentComponent } from '../historie-item-attachment/historie-item-attachment.component';
import { HistorieItemAktenzeichenComponent } from './historie-item-aktenzeichen.component';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { getDataTestClassOf, getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { HistorieItemHeaderComponent } from '../historie-item-header/historie-item-header.component';

describe('HistorieItemAktenzeichenComponent', () => {
  let component: HistorieItemAktenzeichenComponent;
  let fixture: ComponentFixture<HistorieItemAktenzeichenComponent>;

  const kommentarCommand: CommandResource = {
    ...createCommandResource(),
    order: CommandOrder.CREATE_KOMMENTAR,
    body: { item: { text: 'Beispiel text fuer einen Kommentar' } },
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        HistorieItemAktenzeichenComponent,
        MockComponent(ExpansionPanelWithUserComponent),
        MockComponent(HistorieItemAttachmentComponent),
        MockComponent(HistorieItemHeaderComponent),
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(HistorieItemAktenzeichenComponent);
    component = fixture.componentInstance;
    component.command = kommentarCommand;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render non expandable historie item', () => {
    component.aktenzeichenDeleted = true;
    fixture.detectChanges();

    const historieItem = getElementFromFixture(
      fixture,
      getDataTestIdOf('historie-item-aktenzeichen-header'),
    );

    expect(historieItem).toBeInstanceOf(HTMLElement);
  });

  it('should render expandable historie item', () => {
    component.aktenzeichenDeleted = false;
    fixture.detectChanges();

    const aktenzeichenText = getElementFromFixture(
      fixture,
      getDataTestClassOf('aktenzeichen-text'),
    );

    expect(aktenzeichenText).toBeInstanceOf(HTMLElement);
  });

  describe('ngOnInit', () => {
    it('should set headline', () => {
      component.setHeadline = jest.fn();

      component.ngOnInit();

      expect(component.setHeadline).toHaveBeenCalled();
    });

    it('should mark aktenzeichen as deleted', () => {
      component.command.body.aktenzeichen = null;

      component.ngOnInit();

      expect(component.aktenzeichenDeleted).toBeTruthy();
    });

    it('should mark aktenzeichen as NOT deleted', () => {
      component.command.body.aktenzeichen = '1';

      component.ngOnInit();

      expect(component.aktenzeichenDeleted).toBeFalsy();
    });
  });

  describe('setHeadline', () => {
    it('should set headline for order ' + CommandOrder.SET_AKTENZEICHEN, () => {
      component.aktenzeichenDeleted = false;
      component.command = { ...kommentarCommand, order: CommandOrder.SET_AKTENZEICHEN };

      component.setHeadline();

      expect(component.headline).toBe('das Aktenzeichen geändert.');
    });

    it('should set headline for deleted aktenzeichen', () => {
      component.aktenzeichenDeleted = true;

      component.setHeadline();

      expect(component.headline).toBe('das Aktenzeichen gelöscht.');
    });
  });
});
