/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { MockComponent } from 'ng-mocks';
import { HistorieItemHeaderComponent } from '../historie-item-header/historie-item-header.component';
import { HistorieItemForwardingComponent } from './historie-item-forwarding.component';

describe('HistorieItemForwardingComponent', () => {
  let component: HistorieItemForwardingComponent;
  let fixture: ComponentFixture<HistorieItemForwardingComponent>;

  const forwardingCommand: CommandResource = {
    ...createCommandResource(),
    order: CommandOrder.REDIRECT_VORGANG,
    body: { item: {} },
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [HistorieItemForwardingComponent, MockComponent(HistorieItemHeaderComponent)],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(HistorieItemForwardingComponent);
    component = fixture.componentInstance;
    component.command = forwardingCommand;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should set headline for order ' + CommandOrder.REDIRECT_VORGANG, () => {
      component.command = { ...forwardingCommand, order: CommandOrder.REDIRECT_VORGANG };

      component.ngOnInit();

      expect(component.headline).toBe('den Vorgang weitergeleitet.');
    });

    it('should set headline for order' + CommandOrder.FORWARD_SUCCESSFULL, () => {
      component.command = { ...forwardingCommand, order: CommandOrder.FORWARD_SUCCESSFULL };

      component.ngOnInit();

      expect(component.headline).toBe('die Weiterleitung bestätigt.');
    });

    it('should set headline for order' + CommandOrder.FORWARD_FAILED, () => {
      component.command = { ...forwardingCommand, order: CommandOrder.FORWARD_FAILED };

      component.ngOnInit();

      expect(component.headline).toBe('die Weiterleitung widerrufen.');
    });
  });
});
