/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MockComponent } from 'ng-mocks';
import { KommentarListInVorgangComponent } from './kommentar-list-in-vorgang.component';
import { KommentarListItemInVorgangComponent } from './kommentar-list-item-in-vorgang/kommentar-list-item-in-vorgang.component';
import {
  createEmptyStateResource,
  createStateResource,
  EMPTY_ARRAY,
} from '@alfa-client/tech-shared';
import { createKommentarListResource } from '../../../../../kommentar-shared/test/kommentar';
import { KommentarListLinkRel } from '@alfa-client/kommentar-shared';

describe('KommentarListInVorgangComponent', () => {
  let component: KommentarListInVorgangComponent;
  let fixture: ComponentFixture<KommentarListInVorgangComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        KommentarListInVorgangComponent,
        MockComponent(KommentarListItemInVorgangComponent),
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(KommentarListInVorgangComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('getKommentare', () => {
    it('should return empty array if state resource is null', () => {
      component.kommentarListStateResource = null;

      expect(component.getKommentare()).toEqual(EMPTY_ARRAY);
    });

    it('should return empty array if resource is null', () => {
      component.kommentarListStateResource = createEmptyStateResource();

      expect(component.getKommentare()).toEqual(EMPTY_ARRAY);
    });

    it('should return embedded resource', () => {
      const kommentareListResource = createKommentarListResource();
      component.kommentarListStateResource = createStateResource(kommentareListResource);

      expect(component.getKommentare()).toEqual(
        kommentareListResource._embedded[KommentarListLinkRel.KOMMENTAR_LIST],
      );
    });
  });
});
