import { Mock, mock } from '@alfa-client/test-utils';
import { Dialog, DialogConfig } from '@angular/cdk/dialog';
import { TestBed } from '@angular/core/testing';
import { OzgcloudDialogService } from './ozgcloud-dialog.service';

describe('OzgcloudDialogService', () => {
  let service: OzgcloudDialogService;
  let dialog: Mock<Dialog>;

  const component = <any>{ name: 'Component' };
  const dialogData = { id: 'ZumBeispiel' };
  const dialogConfig: DialogConfig = {
    hasBackdrop: true,
    disableClose: true,
  };
  const dialogConfigWithData: DialogConfig = { data: dialogData };
  const viewContainerRef = <any>{};
  const dialogConfigWithDataAndViewContainerRef: DialogConfig = {
    data: dialogData,
    viewContainerRef,
  };
  const dialogConfigWithOwnConfigAndViewContainerRef: DialogConfig = {
    ...dialogConfig,
    viewContainerRef,
  };
  const dialogConfigWithOutDataAndWithViewContainerRef: DialogConfig = {
    viewContainerRef,
  };

  beforeEach(() => {
    dialog = mock(Dialog);

    TestBed.configureTestingModule({
      providers: [
        {
          provide: Dialog,
          useValue: dialog,
        },
      ],
    });
    service = TestBed.inject(OzgcloudDialogService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('open', () => {
    it('should open dialog with data', () => {
      service.open(component, dialogData);

      expect(dialog.open).toHaveBeenCalledWith(component, dialogConfigWithData);
    });

    it('should open dialog wihtout data', () => {
      service.open(component);

      expect(dialog.open).toHaveBeenCalledWith(component, undefined);
    });
  });

  describe('openInCallingComponentContext', () => {
    it('should open dialog with data', () => {
      service.openInCallingComponentContext(component, viewContainerRef, dialogData);

      expect(dialog.open).toHaveBeenCalledWith(component, dialogConfigWithDataAndViewContainerRef);
    });

    it('should open dialog with custom config', () => {
      service.openInCallingComponentContext(component, viewContainerRef, null, dialogConfig);

      expect(dialog.open).toHaveBeenCalledWith(component, dialogConfigWithOwnConfigAndViewContainerRef);
    });

    it('should open dialog without data and custom config', () => {
      service.openInCallingComponentContext(component, viewContainerRef);

      expect(dialog.open).toHaveBeenCalledWith(component, dialogConfigWithOutDataAndWithViewContainerRef);
    });
  });

  describe('openWizard', () => {
    it('should open wizard dialog', () => {
      service.openWizard(component);

      expect(dialog.open).toHaveBeenCalledWith(component, {
        ...service.WIZARD_DIALOG_CONFIG,
      });
    });

    it('should open wizard dialog with data', () => {
      service.openWizard(component, dialogData);

      expect(dialog.open).toHaveBeenCalledWith(component, {
        ...service.WIZARD_DIALOG_CONFIG,
        data: dialogData,
      });
    });
  });

  describe('open full screen dialog', () => {
    it('should call openInCallingComponentContext', () => {
      service.openInCallingComponentContext = jest.fn();

      service.openFullScreenDialog(component, viewContainerRef, dialogData);

      expect(service.openInCallingComponentContext).toHaveBeenCalledWith(
        component,
        viewContainerRef,
        dialogData,
        service.GREY_BLUR_CONFIG,
      );
    });
  });
});
