/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { AuthService } from '@alfa-client/app-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { UserProfileService } from '@alfa-client/user-profile-shared';
import { MockComponent } from 'ng-mocks';
import { UserProfileInHeaderContainerComponent } from './user-profile-in-header-container.component';
import { UserProfileInHeaderComponent } from './user-profile-in-header/user-profile-in-header.component';

describe('UserProfileInHeaderContainerComponent', () => {
  let component: UserProfileInHeaderContainerComponent;
  let fixture: ComponentFixture<UserProfileInHeaderContainerComponent>;

  const authService: Mock<AuthService> = mock(AuthService);
  const userProfileService: Mock<UserProfileService> = mock(UserProfileService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        UserProfileInHeaderContainerComponent,
        MockComponent(UserProfileInHeaderComponent),
      ],
      providers: [
        {
          provide: AuthService,
          useValue: authService,
        },
        {
          provide: UserProfileService,
          useValue: userProfileService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(UserProfileInHeaderContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service', () => {
      component.ngOnInit();

      expect(userProfileService.getCurrentUser).toHaveBeenCalled();
    });
  });

  describe('logout', () => {
    it('should call authService', () => {
      component.logout();

      expect(authService.logout).toHaveBeenCalled();
    });
  });
});
