import { BescheidLinkRel, BescheidService } from '@alfa-client/bescheid-shared';
import { StateResource, createEmptyStateResource, isLoaded } from '@alfa-client/tech-shared';
import { VorgangService, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Resource, hasLink } from '@ngxp/rest';
import { Observable, filter, map, merge } from 'rxjs';
import { BescheidenFormService } from '../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-weiter-button',
  templateUrl: './vorgang-detail-bescheiden-weiter-button.component.html',
})
export class VorgangDetailBescheidenWeiterButtonComponent implements OnInit {
  @Input() submitResource: StateResource<Resource> = createEmptyStateResource();
  @Output() clickEmitter: EventEmitter<MouseEvent> = new EventEmitter<MouseEvent>();

  showButton$: Observable<boolean>;

  constructor(
    public formService: BescheidenFormService,
    private readonly bescheidService: BescheidService,
    private readonly vorgangService: VorgangService,
  ) {}

  ngOnInit(): void {
    this.showButton$ = merge(
      this.vorgangService.getVorgangWithEingang(),
      this.bescheidService.getBescheidDraftIfExists(),
    ).pipe(
      filter(isLoaded),
      map(
        (stateResource) =>
          hasLink(stateResource.resource, VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT) ||
          hasLink(stateResource.resource, BescheidLinkRel.UPDATE),
      ),
    );
  }
}
