/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AfterContentInit, Component } from '@angular/core';
import { StateResource, createEmptyStateResource } from '@alfa-client/tech-shared';
import {
  WiedervorlageListResource,
  WiedervorlageResource,
  WiedervorlageService,
} from '@alfa-client/wiedervorlage-shared';
import { Observable, of, startWith } from 'rxjs';
import { WiedervorlageFormService } from './wiedervorlage-page/wiedervorlage-form/wiedervorlage.formservice';

@Component({
  selector: 'alfa-wiedervorlage-page-container',
  templateUrl: './wiedervorlage-page-container.component.html',
  styleUrls: ['./wiedervorlage-page-container.component.scss'],
  providers: [WiedervorlageFormService],
})
export class WiedervorlagePageContainerComponent implements AfterContentInit {
  wiedervorlageListStateResource$: Observable<StateResource<WiedervorlageListResource>>;
  wiedervorlageStateResource$: Observable<StateResource<WiedervorlageResource>>;

  constructor(private wiedervorlageService: WiedervorlageService) {}

  ngAfterContentInit(): void {
    this.wiedervorlageListStateResource$ = this.wiedervorlageService.getWiedervorlageList();
    this.wiedervorlageStateResource$ = this.getWiedervorlage().pipe(
      startWith(createEmptyStateResource<WiedervorlageResource>(true)),
    );
  }

  getWiedervorlage(): Observable<StateResource<WiedervorlageResource>> {
    if (this.wiedervorlageService.isNewWiedervorlage()) {
      return of(createEmptyStateResource<WiedervorlageResource>());
    } else {
      return this.wiedervorlageService.getWiedervorlage();
    }
  }
}
