import { EventData, Mock, getMockComponent, mock, triggerEvent } from '@alfa-client/test-utils';
import { OrganisationsEinheitResource } from '@alfa-client/zustaendige-stelle-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { faker } from '@faker-js/faker';
import { Resource } from '@ngxp/rest';
import { InstantSearchComponent, InstantSearchResult } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { createInstantSearchResult } from '../../../../../design-system/src/test/search';
import { getDataTestIdOf } from '../../../../../tech-shared/test/data-test';
import { createOrganisationsEinheitResource } from '../../../../../zustaendige-stelle-shared/test/organisations-einheit';
import { SearchZustaendigeStelleFormservice } from '../search-zustaendige-stelle.formservice';
import { SearchZustaendigeStelleFormComponent } from './search-zustaendige-stelle-form.component';

describe('SearchZustaendigeStelleFormComponent', () => {
  let component: SearchZustaendigeStelleFormComponent<Resource>;
  let fixture: ComponentFixture<SearchZustaendigeStelleFormComponent<Resource>>;

  const searchComp: string = getDataTestIdOf('search');

  const formService: Mock<SearchZustaendigeStelleFormservice> = mock(SearchZustaendigeStelleFormservice);

  const instantSearchResult: InstantSearchResult<Resource> = createInstantSearchResult<OrganisationsEinheitResource>();
  const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ReactiveFormsModule],
      declarations: [SearchZustaendigeStelleFormComponent, MockComponent(InstantSearchComponent)],
      providers: [
        {
          provide: SearchZustaendigeStelleFormservice,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(SearchZustaendigeStelleFormComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('instant search component', () => {
    it('should be called with search results', () => {
      component.searchResults = [instantSearchResult];

      fixture.detectChanges();

      expect(getInstantSearchComponent().searchResults).toEqual([instantSearchResult]);
    });

    function getInstantSearchComponent(): InstantSearchComponent {
      return getMockComponent<InstantSearchComponent>(fixture, InstantSearchComponent);
    }

    it('should emit selected search result on searchResultSelected output', () => {
      const selectSearchResultSpy: jest.SpyInstance = (component.selectSearchResult.emit = jest.fn());
      const eventData: EventData<SearchZustaendigeStelleFormComponent<Resource>> = {
        fixture,
        elementSelector: searchComp,
        name: 'searchResultSelected',
        data: { data: organisationsEinheitResource },
      };

      triggerEvent(eventData);

      expect(selectSearchResultSpy).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should emit search on searchQueryChanged output', () => {
      const searchSpy: jest.SpyInstance = (component.search.emit = jest.fn());
      const searchBy: string = faker.lorem.word();
      const eventData: EventData<SearchZustaendigeStelleFormComponent<Resource>> = {
        fixture,
        elementSelector: searchComp,
        name: 'searchQueryChanged',
        data: { searchBy },
      };

      triggerEvent(eventData);

      expect(searchSpy).toHaveBeenCalledWith(searchBy);
    });

    describe('should emit clear search result', () => {
      let clearSearchResultSpy: jest.SpyInstance;
      let eventData: EventData<SearchZustaendigeStelleFormComponent<Resource>>;

      beforeEach(() => {
        clearSearchResultSpy = component.clearSearchResult.emit = jest.fn();
        eventData = {
          fixture,
          elementSelector: searchComp,
          name: 'TBD',
          data: { searchBy: faker.lorem.word() },
        };
      });

      it('on searchClosed output', () => {
        eventData = { ...eventData, name: 'searchClosed' };

        triggerEvent(eventData);

        expect(clearSearchResultSpy).toHaveBeenCalledWith();
      });

      it('on searchQueryCleared output', () => {
        eventData = { ...eventData, name: 'searchClosed' };

        triggerEvent(eventData);

        expect(clearSearchResultSpy).toHaveBeenCalled();
      });
    });
  });
});
