import { dispatchEventFromFixture, Mock, mock } from '@alfa-client/test-utils';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { Resource } from '@ngxp/rest';
import { ButtonComponent, SearchIconComponent } from '@ods/system';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { SearchZustaendigeStelleDialogComponent } from '../search-zustaendige-stelle-dialog/search-zustaendige-stelle-dialog.component';
import { ZustaendigeStelleButtonComponent } from './zustaendige-stelle-button.component';

describe('ZustaendigeStelleButtonComponent', () => {
  let component: ZustaendigeStelleButtonComponent<Resource>;
  let fixture: ComponentFixture<ZustaendigeStelleButtonComponent<Resource>>;

  const title: string = faker.lorem.words(3);
  const dataTestId: string = faker.lorem.word();

  const button: string = getDataTestClassOf('zustaendige-stelle-button');

  let dialogService: Mock<OzgcloudDialogService>;

  beforeEach(async () => {
    dialogService = mock(OzgcloudDialogService);
    await TestBed.configureTestingModule({
      declarations: [ZustaendigeStelleButtonComponent],
      imports: [MockComponent(SearchIconComponent), MockComponent(ButtonComponent)],
      providers: [
        {
          provide: OzgcloudDialogService,
          useValue: dialogService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(ZustaendigeStelleButtonComponent);
    component = fixture.componentInstance;
    component.title = title;
    component.dataTestId = dataTestId;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('search zustaendige stelle button', () => {
    it('should call openSearchDialog', () => {
      component.openSearchDialog = jest.fn();

      fixture.detectChanges();
      dispatchEventFromFixture(fixture, button, 'clickEmitter');

      expect(component.openSearchDialog).toHaveBeenCalled();
    });
  });

  describe('open search dialog', () => {
    it('should call dialog service', () => {
      component.openSearchDialog();

      expect(dialogService.openFullScreenDialog).toHaveBeenCalledWith(
        SearchZustaendigeStelleDialogComponent,
        component.viewContainerRef,
        title,
      );
    });
  });
});
