package de.ozgcloud.alfa.hint;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;

@Service
class HintService {

	static final String USER_NO_ORGANISATIONS_EINHEIT_ID_CODE = "user.no.organisationsEinheitId";

	@Autowired
	private CurrentUserService currentUserService;

	public boolean hasHints() {
		return !findHints().isEmpty();
	}

	public List<Hint> findHints() {
		var hints = new ArrayList<Hint>();

		getHasUserNoOrganisationsEinheitIdHint().ifPresent(hints::add);

		return hints;
	}

	Optional<Hint> getHasUserNoOrganisationsEinheitIdHint() {
		if (getOrganisationsEinheitIds().isEmpty() && hasOrganisationsEinheitIdRequiredRole()) {
			return Optional.of(buildUserNoOrganisationsEinheitIdHint());
		}
		return Optional.empty();
	}

	private Hint buildUserNoOrganisationsEinheitIdHint() {
		return Hint.builder().code(USER_NO_ORGANISATIONS_EINHEIT_ID_CODE).build();
	}

	Collection<String> getOrganisationsEinheitIds() {
		return currentUserService.getUser().getOrganisationseinheitIds();
	}

	boolean hasOrganisationsEinheitIdRequiredRole() {
		return !currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER) && !currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE);
	}
}