package de.ozgcloud.alfa.resource;

import java.util.Collection;
import java.util.Optional;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.alfa.common.errorhandling.ResourceNotFoundException;
import de.ozgcloud.common.errorhandling.TechnicalException;
import jakarta.validation.constraints.NotBlank;
import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(OzgcloudResourceController.PATH)
@RequiredArgsConstructor
public class OzgcloudResourceController {

	static final String PATH = "/api/resources";
	static final String PARAM_URI = "uri";

	private final Collection<OzgcloudResourceURIResolver> resolvers;
	private final OzgcloudResourceModelAssembler assembler;

	@GetMapping(params = PARAM_URI)
	public RepresentationModel<EntityModel<OzgcloudResource>> getOzgcloudResource(@RequestParam @NotBlank String uri) {
		var resourceLink = resolveUri(uri);
		if (resourceLink.isEmpty()) {
			throw new ResourceNotFoundException(OzgcloudResource.class, uri);
		}
		return assembler.toModel(new OzgcloudResourceURIResolveResult(uri, resourceLink.get()));
	}

	private Optional<Link> resolveUri(String uri) {
		return resolvers.stream()
				.map(resolver -> resolver.resolve(uri))
				.flatMap(Optional::stream)
				.reduce((r1, r2) -> {
					throw new TechnicalException("Multiple resolvers accepted uri " + uri);
				});
	}
}
