/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.time.LocalDateTime;
import java.time.ZoneOffset;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.boot.info.BuildProperties;
import org.springframework.hateoas.EntityModel;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

class RootControllerTest {

	@Spy
	@InjectMocks
	private RootController controller;
	@Mock
	private BuildProperties properties;
	@Mock
	private RootModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void mock() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();

		when(modelAssembler.toModel(any())).thenAnswer(i -> EntityModel.of(i.getArguments()[0]));
	}

	@DisplayName("Root")
	@Nested
	class TestRoot {

		@Test
		void shouldHaveJavaVersion() throws Exception {
			doRequest().andExpect(jsonPath("$.javaVersion").exists());
		}

		@Test
		void shouldHaveVersion() throws Exception {
			when(properties.getVersion()).thenReturn("42");

			doRequest().andExpect(jsonPath("$.version").value("42"));
		}

		@Test
		void shouldHaveBuildTime() throws Exception {
			when(properties.getTime()).thenReturn(LocalDateTime.parse("2021-04-01T10:30").toInstant(ZoneOffset.UTC));

			doRequest().andExpect(jsonPath("$.buildTime").exists());
		}

		@Test
		void shouldHavelProduction() throws Exception {
			doRequest().andExpect(jsonPath("$.production").exists());
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(RootController.PATH)).andExpect(status().isOk());
		}
	}
}