package de.ozgcloud.alfa.bescheid;

import static de.ozgcloud.alfa.bescheid.DocumentModelAssembler.*;
import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;

class DocumentModelAssemblerTest {

	@Spy
	private DocumentModelAssembler assembler;

	@Nested
	class TestToModel {

		private final Document document = DocumentTestFactory.create();

		@Test
		void shouldHaveSelfLink() {
			var model = callToModel();

			assertThat(model.getLink(IanaLinkRelations.SELF)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(DocumentController.PATH + "/" + DocumentTestFactory.ID);
		}

		@Test
		void shouldHaveFileIdLink() {
			var model = callToModel();

			assertThat(model.getLink(REL_FILE)).isPresent().get().extracting(Link::getHref)
					.isEqualTo("/api/binaryFiles/" + DocumentTestFactory.FILE_ID);
		}

		private EntityModel<Document> callToModel() {
			return assembler.toModel(document);
		}
	}
}
