package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;

class FachstelleModelAssemblerTest {

	@Spy
	private FachstelleModelAssembler assembler;

	@Nested
	class TestToModel {

		@Test
		void shouldHaveSelfLink() {
			var model = assembler.toModel(FachstelleTestFactory.create());

			assertHaveSelfLink(model);
		}

		private void assertHaveSelfLink(EntityModel<Fachstelle> model) {
			assertThat(model.getLink(IanaLinkRelations.SELF_VALUE)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(FachstelleController.PATH + "/" + FachstelleTestFactory.ID);
		}
	}

	@Nested
	class TestToCollectionModel {

		private Fachstelle fachstelle = FachstelleTestFactory.create();

		@Test
		void shouldCallToModel() {
			callAssembler();

			verify(assembler).toModel(fachstelle);
		}

		@Test
		void shouldHaveSelfLink() {
			var model = callAssembler();

			assertThat(model.getLink(IanaLinkRelations.SELF_VALUE)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(FachstelleController.PATH);
		}

		@Test
		void shouldContainEntityModel() {
			var entityModel = EntityModel.of(fachstelle);
			doReturn(entityModel).when(assembler).toModel(fachstelle);

			var model = callAssembler();

			assertThat(model).containsExactly(entityModel);
		}

		private CollectionModel<EntityModel<Fachstelle>> callAssembler() {
			return assembler.toCollectionModel(List.of(fachstelle));
		}
	}
}
