package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import com.thedeanda.lorem.LoremIpsum;

class OrganisationsEinheitServiceTest {

	@InjectMocks
	private OrganisationsEinheitService service;

	@Mock
	private OrganisationsEinheitRemoteService remoteService;

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(OrganisationsEinheitTestFactory.ID);

			verify(remoteService).getById(OrganisationsEinheitTestFactory.ID);
		}

		@Test
		void shouldReturnOrganisationsEinheit() {
			var organisationsEinheit = OrganisationsEinheitTestFactory.create();
			when(remoteService.getById(OrganisationsEinheitTestFactory.ID)).thenReturn(organisationsEinheit);

			var returnedOrganisationsEinheit = service.getById(OrganisationsEinheitTestFactory.ID);

			assertThat(returnedOrganisationsEinheit).isEqualTo(organisationsEinheit);
		}
	}

	@Nested
	class TestSearchOrganisationsEinheiten {

		private final String searchBy = LoremIpsum.getInstance().getWords(5);

		@Test
		void shouldCallRemoteService() {
			callService();

			verify(remoteService).search(searchBy);
		}

		@Test
		void shouldReturnOrganisationsEinheiten() {
			var organisationsEinheitHeader = OrganisationsEinheitHeaderTestFactory.create();
			when(remoteService.search(searchBy)).thenReturn(Stream.of(organisationsEinheitHeader));

			var organisationsEinheitHeaders = callService();

			assertThat(organisationsEinheitHeaders).containsExactly(organisationsEinheitHeader);
		}

		private Stream<OrganisationsEinheitHeader> callService() {
			return service.searchOrganisationsEinheiten(searchBy);
		}
	}

}
