/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.UUID;

import org.springframework.mock.web.MockMultipartFile;
import org.springframework.util.unit.DataSize;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.file.GrpcOzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcFindFilesResponse;

public class BinaryFileTestFactory {

	public static final String ID = UUID.randomUUID().toString();
	public static final FileId FILE_ID = FileId.from(ID);
	public static final String NAME = "TestFile";
	public static final String CONTENT_TYPE = "application/image";
	public static final byte[] DATA = "juhu, ein bild".getBytes();
	public static final MockMultipartFile TEST_FILE = new MockMultipartFile("file", NAME, CONTENT_TYPE, DATA);
	public static final long SIZE = TEST_FILE.getSize();

	public static final String FIELD = "attachment";
	public static final String VORGANG_ID = VorgangHeaderTestFactory.ID;
	static final InputStream STREAM = new ByteArrayInputStream(DATA);

	public static GrpcFindFilesResponse createFindBinaryFilesResponse() {
		return GrpcFindFilesResponse.newBuilder().addFile(GrpcOzgFileTestFactory.create()).build();
	}

	public static byte[] createByteOfSize(DataSize size) {
		var testStr = LoremIpsum.getInstance().getWords(1);
		var contentSize = (int) size.toBytes();
		var result = new byte[contentSize];

		System.arraycopy(testStr.getBytes(), 0, result, contentSize - testStr.length(), testStr.length());

		return result;
	}
}