package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Iterator;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatcher;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.archive.grpc.export.ExportServiceGrpc.ExportServiceBlockingStub;
import de.ozgcloud.archive.grpc.export.GrpcExportVorgangRequest;
import de.ozgcloud.archive.grpc.export.GrpcExportVorgangResponse;

class ExportRemoteServiceTest {

	@Spy
	@InjectMocks
	private ExportRemoteService service;
	@Mock
	private ExportServiceBlockingStub exportServiceStub;

	@Nested
	class TestExportVorgang {

		public static final String VORGANG_ID = VorgangHeaderTestFactory.ID;

		private static final ArgumentMatcher<GrpcExportVorgangRequest> HAS_VORGANG_ID = request -> request.getVorgangId().equals(VorgangHeaderTestFactory.ID);

		@Mock
		private Iterator<GrpcExportVorgangResponse> responseIterator;
		private MockedStatic<StreamedExportedVorgangFile> mockedStaticExportedFile;

		@BeforeEach
		void init() {
			mockedStaticExportedFile = mockStatic(StreamedExportedVorgangFile.class);
			mockedStaticExportedFile.when(() -> StreamedExportedVorgangFile.getFileNameFrom(responseIterator)).thenReturn(GrpcFileTestFactory.FILE_NAME);
			when(exportServiceStub.exportVorgang(any(GrpcExportVorgangRequest.class))).thenReturn(responseIterator);
		}

		@AfterEach
		void cleanup() {
			mockedStaticExportedFile.close();
		}

		@Test
		void shouldExportVorgang() {
			service.exportVorgang(VORGANG_ID);

			verify(exportServiceStub).exportVorgang(argThat(HAS_VORGANG_ID));
		}

		@Test
		void shouldReturnExportedVorgangFile() {
			var exportedVorgangFile = service.exportVorgang(VORGANG_ID);

			assertThat(exportedVorgangFile).isInstanceOf(StreamedExportedVorgangFile.class).extracting("responseIterator").isEqualTo(responseIterator);
		}
	}
}
